from typing import Annotated, NamedTuple

from elefantolib_fastapi import exceptions
from elefantolib_fastapi.requests import Request

from fastapi import HTTPException, Header, status


class CommonHeaders(NamedTuple):
    auth_token: Annotated[str, Header(alias='Authorization')]
    correlation_id: Annotated[str, Header(alias='X-Correlation-Id')]
    locale: Annotated[str, Header(alias='Accept-Language')]


def request_depend(request: Request):
    try:
        request.pfm.validate()
    except Exception as e:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail=str(e))


async def is_authenticated(request: Request):
    if not request.pfm.user.is_authenticated():
        raise exceptions.Unauthorized



from typing import Any

from fastapi import HTTPException, status


class Unauthorized(HTTPException):

    def __init__(
        self, status_code: int = status.HTTP_404_NOT_FOUND,
        detail: Any = 'Unauthorized',
        headers: dict | None = None,
    ) -> None:
        super().__init__(status_code=status_code, detail=detail, headers=headers)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['elefantolib_fastapi']

package_data = \
{'': ['*']}

install_requires = \
['elefantolib>=0.11.2,<0.12.0',
 'fastapi>=0.95.2,<0.96.0',
 'redis>=4.5.5,<5.0.0']

setup_kwargs = {
    'name': 'elefantolib-fastapi',
    'version': '0.3.1',
    'description': '',
    'long_description': '## Elefantolib for FastAPI\n\n> **_NOTE:_**  Only for this library developers. After clone this repository you should run command:\n> \n\n ```console \ngit config core.hooksPath .githooks\n```\n\n\n## Installation\n\n<div class="termy">\n\n```console\npoetry add elefantolib-fastapi\n```\n</div>\n\n## Example\n\n### Prepare\n\n* Add environmental variables\n\n```\nSECRET=\nALGORITHM=\nISSUER=\n```\n* Defaults:\n    \n    - SECRET - not set, this is required\n    - ALGORITHM=HS256\n    - ISSUER=Consumer\n\n### Create it\n\n* Create a file `main.py` with:\n\n```Python\nfrom elefantolib_fastapi.requests import Request\nfrom elefantolib_fastapi.routes import APIRoute\n\nfrom fastapi import FastAPI\n\napp = FastAPI()\n\napp.router.route_class = APIRoute\n\n\n@app.get(\'/\')\ndef index(request: Request):\n    # TODO something\n    response = request.pfm.services.some_service_name.get(\'path-to-endpoint\')\n    return response\n\n```',
    'author': 'gglassota',
    'author_email': 'gglassota2@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import os
from typing import Callable

from elefantolib import context
from elefantolib.provider import fastapi_provider

import fastapi
from fastapi import routing

from redis import asyncio as aioredis

from .requests import Request


class APIRoute(routing.APIRoute):

    def get_route_handler(self) -> Callable:
        original_route_handler = super().get_route_handler()

        async def custom_route_handler(request: fastapi.Request) -> fastapi.Response:
            pvr = fastapi_provider.FastAPIProvider(request=request)
            request.scope['pfm'] = context.AsyncPlatformContext(pvr=pvr)

            if redis_url := os.environ.get('REDIS_URL'):
                request.scope['redis'] = aioredis.from_url(redis_url)

            new_request = Request(request.scope, request.receive)
            return await original_route_handler(new_request)
        return custom_route_handler



from elefantolib.context import AsyncPlatformContext

from fastapi import requests

from redis.asyncio.client import Redis


class Request(requests.Request):

    @property
    def pfm(self) -> AsyncPlatformContext:
        pfm = self.scope['pfm']
        pfm.validate()
        return pfm

    @property
    def redis(self) -> Redis:
        return self.scope.get('redis')
